% Monte Carlo demo for Multivariate non-Gaussian synthesis based on exact covariance
% mapping

% choose the transform type (see documentation of MapCovar)
name = 'LN';
%name = 'Gaussian';
params = [0 1]; % parameters for transform

% length of synthesized series
N = 2^12; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate covariance structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % set parameters for the FARIMA(0,D,0) model
W = [1 0.1;0 1];
d = [0.3 0.2];
Sigmae = eye(2);

% generate targeted covariance structure
Ncov = N+1; % cov. could be asymmetric, get cov. seq. of length N+1
RY = CovarFARIMA0D0(Ncov,d,W,Sigmae);

% normalize according to transform
RY = NormalizeCovar(RY,name,params);
% Map RY to RX
[RX,K,funch] = MapCovar(RY,name,params);
% NOTE: RX is not necessarily a valid covariance structure 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initalize
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization step for multivariate Gaussian synthesis
tic;
[Rxapprox,approxFlag,BmArray] = InitStepMultivarGaussBestApprox(RX,N);
%[Rxapprox,approxFlag,BmArray] = InitMGBestApproxLapack(RX,N); % faster initialization using mex and LAPACK
initTime = toc
if approxFlag,
    disp('Approximation was needed to ensure psd of inverted target covariance.');
else
    disp('Inverted target covariance was psd - no approximation needed.');
end

P = length(RX); % number of components in the series
X = cell(1,P);
% Initialize X
for p=1:P,
    X{p} = zeros(K,N);
end
Y = zeros(P,N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate series
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
numRealizations = 10; % number of Monte Carlo realizations

tic;
for n=1:numRealizations,

    for kk=1:floor(K/2),
        [Xcopy1,Xcopy2] = SynthStepMultivarGaussBestApprox(BmArray,N);        
        % Xcopy1 and Xcopy2 are independent copies of the multivariate Gaussian process 
        for p=1:P,
            X{p}(2*kk-1,:) = Xcopy1(p,:);
            X{p}(2*kk,:) = Xcopy2(p,:);        
        end
    end
    if rem(K,2)>0,
        % add one more component since K is odd
        [Xcopy1,Xcopy2] = SynthStepMultivarGaussBestApprox(BmArray,N);        
        for p=1:P,
            X{p}(K,:) = Xcopy1(p,:);
        end
    end
    % Transform Gaussian series
    for p=1:P,
        Y(p,:) = funch(X{p});
    end
    % Y(1,:),...,Y(P,:) give a realization of the multivariate series

    % Process Y(p,:)...
end
toc