function [a] = reversion(A,zeropadLength)
% Calculates coefficients for series reversion.
%
% Usage:
%  [a] = reversion(A,zeropadLength)
% 
% Inputs:
%   A			input coefficient sequence to be reversed
%   zeropadLength	length of trailing coeffients to set to 0,
%			i.e., the coefficient series to be reversed is
%			[A zeros(1,zeropadLength)]
%                       In cases where the length of coefficient sequence
%                       is short, this will provide greater accuracy for the reversion series.
%			if omitted, the value will be set to 0
% Output
%   a		coefficient sequence obtained by reversion from A,
%		row vector of length lenght(A)+zeropadLength
%
% Note: this use calculations of reversion through matrix on p. 47 in 
% Henrici (1974, volume 1)
%
% Copyright (c) 2010, Vladas Pipiras UNC, Hannes Helgason, ENS de Lyon

if nargin < 2,
  % set default zeropad length
  zeropadLength = 0;
end

% force A to be a column vector
A = reshape(A,length(A),1);
% zeropad
A = [A; zeros(zeropadLength,1)];

% Find the coefficients for the reversion
K = length(A) ;

A_matrix = A*ones(1,K) ;
A_matrix = [A_matrix; zeros(1,K)] ;
A_matrix = reshape(A_matrix,K,K+1) ;
A_matrix = tril(A_matrix) ;
A_matrix = A_matrix' ;
A_matrix = A_matrix(1:K,:) ;

a_matrix = A' ;
A_matrix_power = A_matrix ;
for k=2:K
    A_matrix_power = A_matrix_power*A_matrix ;
    a_matrix = [a_matrix; [zeros(1,k-1) A_matrix_power(1,1:K-k+1)]] ;
end ;

a_matrix = a_matrix^(-1) ;
a = a_matrix(1,:) ; 
