#include "mex.h"
#include "HermiteLib.h"

/* mex interface to evalHermitePoly */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* Declarations */
    const mxArray *xPtr;
    const mxArray *normalizeFlagPtr;
    double *x, *outArray;
    int xNumRows, xNumCols, N;
    int k,j;
    int normalizeFlag;
    
    /* Get input data */
    xPtr = prhs[0];
    x = mxGetPr(xPtr);
    xNumRows = mxGetM(xPtr);
    xNumCols = mxGetN(xPtr);
    
    N = (int) mxGetScalar(prhs[1]);
    
    /* setting normalize flag to 0 by default */
	normalizeFlag = 0;
	if (nrhs > 2) {
        /* flag was provided */
        normalizeFlagPtr = prhs[2];
        if (mxIsLogicalScalar(normalizeFlagPtr)) {
            /* input is a logical value */
            if (mxIsLogicalScalarTrue(normalizeFlagPtr)) {
                normalizeFlag = 1;
            }
        } else {
            /* assuming that the input is a scalar */
            normalizeFlag = (int) mxGetScalar(normalizeFlagPtr);
        }
	}
    
    /* Allocate memory and assign output pointer (mxReal is our data-type) */
    plhs[0] = mxCreateDoubleMatrix(xNumRows, xNumCols, mxREAL);
    outArray = mxGetPr(plhs[0]);

    
    /* loop over rows */
	for(k=0;k<xNumRows;k++) {
        /* loop over columns */
        for(j=0;j<xNumCols;j++) {
            outArray[(k*xNumCols)+j] = evalHermitePoly(x[(k*xNumCols)+j], N, normalizeFlag);
        }
    }
    return;
}

